public class MaxProduct{
    private static int getRandomNumber(int min, int max) {
        return (int) ((Math.random() * (max - min)) + min);
    }
    
    private static void printArray(int [] array){
        for (int element: array) {
            System.out.println(element+" ");
        }
    }
    
    public static int maxProductNaive(int[] t) {
        int best = 0;
        int n = t.length;
        for (int i = 0; i < n -1; i++) {
            for (int j = i + 1; j < n; j++) { 
                if (t[i] * t[j] > best)
                    best =  t[i] * t[j];
            }
        }

        return best;
    }

    public static int maxProductFast(int[] t) {
        int index = 0;
        int n = t.length;
    
        for (int i = 0; i < n; i++) {
            if (t[i] > t[index])
                index = i;
        }
        int temp = t[index];
        t[index] = t[n - 1];
        t[n - 1] = temp;
    
        index = 0;
        for (int j = 0; j < n - 1; j++) {
            if (t[j] > t[index])
                index = j;
        }
        temp = t[index];
        t[index] = t[n - 2];
        t[n - 2] = temp;
    
        return t[n - 1] * t[n - 2];   
    }

    public static int stressTest(int repeats, int N, int M) {
        // N - total number of list elements
        // M - each element is random number from 0 to M
        int ok_count = 0;
  

        for (int i = 0; i < repeats; i++) {
            int arr_size = getRandomNumber(2,N); //rand number from 2 to N
            int[] a = new int[arr_size];
            for (int j = 0; j < arr_size; j++) {
                int x = getRandomNumber(0,M); //rand number from 0 to M
                a[j] = x;
            }
    
            int sol_naive = maxProductNaive(a);
            int sol_linear = maxProductFast(a);
            
    
            if (sol_naive == sol_linear) {
                ok_count++;
            }
            else {
                System.out.println("Error after iteration " + ok_count);
                printArray(a);
                System.out.println("linear produced "+ sol_linear +
                                        " and naive produced " + sol_naive);
                return 1;
            }
        }
        System.out.println(ok_count + " tests passed");
        return 0;
    }
	
	public static int performanceTest(int num_tests, int min_len, int max_len, int step){	
		long start;
		long end;
		long time_spent;
		
		// generate random inputs of length N in a given range
		for (int N = min_len; N < max_len+step; N+=step){
			int[] a = new int[N];
			for (int j = 0; j < N; j++) {
				int x = getRandomNumber(0,100); //rand number from 0 to 100
				a[j] = x;
			}
			
			long total_spent_naive = 0;
			long total_spent_fast = 0;
			for (int i=0; i<num_tests; i++){			
				start = System.nanoTime();
				int sol_naive = maxProductNaive(a);
				end = System.nanoTime();
				time_spent = end - start;
				total_spent_naive += time_spent;				
				
				start = System.nanoTime();
				int sol_fast = maxProductFast(a);
				end = System.nanoTime();
				time_spent = end - start;
				total_spent_fast += time_spent;
			}       

			// Print input size, runtimes
			System.out.print(N+"\t");
			System.out.print((total_spent_naive/num_tests)+"\t"); //divide by 1000000 to get milliseconds.
			System.out.println(total_spent_fast/num_tests); //divide by 1000000 to get milliseconds.
		}
		return 0;       
	}
    
    public static void main(String [] args){
        MaxProduct.stressTest(10,6,12);
        MaxProduct.performanceTest(5, 500, 5000, 500);
    }
}